#include "IZ_net.h"

bool IZ_NetIsConnected(IZ_NetState* state) {
	if (state->ws.interrupted) {
		return false;
	}
	if (!state->ws.context) {
		return false;
	}
	if (!state->ws.connection) {
		return false;
	}
	return true;
}

void IZ_NetLoadConfig(IZ_NetState* state, const char* config_path) {
	char buffer[32];

	ini_gets("Network", "Username", IZ_NET_DEFAULT_STATE.config.username, buffer, 32, config_path);
	memcpy_s(state->config.username, 32, buffer, 32);

	state->config.interval = ini_getl("Network", "Interval", IZ_NET_DEFAULT_STATE.config.interval, config_path);
}

IZ_ProcedureResult IZ_NetSaveConfig(IZ_NetState* state, const char* config_path) {
	if (!ini_puts("Network", "Username", state->config.username, config_path)) {
		return -1;
	}

	if (!ini_putl("Network", "Interval", state->config.interval, config_path)) {
		return -1;
	}

	return 0;
}

void IZ_NetOverrideConfig(IZ_NetState* state, u8 argc, const char* argv[]) {
	const char* cmdline_buffer;
	if ((cmdline_buffer = IZ_ConfigGetCommandlineOption(argc, argv, "-i"))) {
		state->config.interval = atoi(cmdline_buffer);
	}
}

IZ_ProcedureResult IZ_NetInitialize(
	IZ_NetState* state,
	void* user_data,
	void* callback,
	const char* config_path,
	u8 argc,
	const char* argv[]
) {
	memcpy_s(state, sizeof(IZ_NetState), &IZ_NET_DEFAULT_STATE, sizeof(IZ_NetState));
	IZ_NetLoadConfig(state, config_path);
	if (IZ_NetSaveConfig(state, config_path) < 0) {
		return -1;
	}
	IZ_NetOverrideConfig(state, argc, argv);
	if (!user_data) {
		return -2;
	}
	state->ws.user_data = user_data;
	state->callback = callback;
	u8 player_index;
	for (player_index = 0; player_index < IZ_PLAYERS; player_index += 1) {
		state->action[player_index] = 0;
	}

	return 0;
}

void IZ_NetConnect(IZ_NetState* state, IZ_WSClientInitializeParams params) {
	if (IZ_NetIsConnected(state)) {
		return;
	}

	if (!state->callback) {
		return;
	}

	state->params = params;
	state->client_thread = SDL_CreateThread(state->callback, "networking", state->ws.user_data);
	SDL_DetachThread(state->client_thread);
}

void IZ_NetDisconnect(IZ_NetState* state) {
	if (!IZ_NetIsConnected(state)) {
		return;
	}
	IZ_WSClientCancelService(&state->ws);
}

void IZ_NetSendBinaryMessage(IZ_NetState* state, void* in, size_t len) {
	if (!IZ_NetIsConnected(state)) {
		return;
	}

	IZ_WSClientVHostData* vhd = (IZ_WSClientVHostData*) lws_protocol_vh_priv_get(
		lws_get_vhost(state->ws.connection),
		lws_get_protocol(state->ws.connection)
	);

	IZ_WebsocketMessage amsg;
	i32 result;
	result = IZ_WebsocketCreateBinaryMessage(state->ws.connection, &amsg, in, len);

	if (result < 0) {
		return;
	}

	lws_ring_insert(vhd->ring, &amsg, 1);
	lws_callback_on_writable(state->ws.connection);
}

void IZ_NetSendTextMessage(IZ_NetState* state, char* in, size_t len) {
	if (!IZ_NetIsConnected(state)) {
		return;
	}

	IZ_WSClientVHostData* vhd = (IZ_WSClientVHostData*) lws_protocol_vh_priv_get(
		lws_get_vhost(state->ws.connection),
		lws_get_protocol(state->ws.connection)
	);

	IZ_WebsocketMessage amsg;
	i32 result;
	result = IZ_WebsocketCreateTextMessage(state->ws.connection, &amsg, in, len);

	if (result < 0) {
		return;
	}

	lws_ring_insert(vhd->ring, &amsg, 1);
	lws_callback_on_writable(state->ws.connection);
}
