#include "IZ_video.h"

IZ_ProcedureResult IZ_VideoSaveConfig(IZ_VideoState* state, const char* config_path) {
	if (!ini_putl("Video", "Width", state->config.width, config_path)) {
		return -1;
	}
	if (!ini_putl("Video", "Height", state->config.height, config_path)) {
		return -1;
	}
	if (!ini_putl("Video", "MaxFps", state->config.max_fps, config_path)) {
		return -1;
	}

	return 0;
}

void IZ_VideoLoadConfig(IZ_VideoState* state, const char* config_path) {
	state->config.width = ini_getl("Video", "Width", IZ_DEFAULT_VIDEO_STATE.config.width, config_path);
	state->config.height = ini_getl("Video", "Height", IZ_DEFAULT_VIDEO_STATE.config.height, config_path);
	state->config.max_fps = ini_getl("Video", "MaxFps", IZ_DEFAULT_VIDEO_STATE.config.max_fps, config_path);
}

void IZ_VideoOverrideConfig(IZ_VideoState* state, u8 argc, const char* argv[]) {
	const char* cmdline_buffer;
	if ((cmdline_buffer = IZ_ConfigGetCommandlineOption(argc, argv, "-f"))) {
		state->config.max_fps = atoi(cmdline_buffer);
	}
}

IZ_ProcedureResult IZ_VideoInitialize(IZ_VideoState* state, void* user_data, const char* config_path, u8 argc, const char* argv[]) {
	SDL_memcpy(state, &IZ_DEFAULT_VIDEO_STATE, sizeof(IZ_VideoState));

	IZ_VideoLoadConfig(state, config_path);
	if (IZ_VideoSaveConfig(state, config_path)) {
//		fprintf_s(stderr, "Error committing video config.\n");
	}
	IZ_VideoOverrideConfig(state, argc, argv);
	state->last_update_at = 0u;
	state->user_data = user_data;

	SDL_Window* window = SDL_CreateWindow(
		IZ_APP_NAME,
		SDL_WINDOWPOS_CENTERED,
		SDL_WINDOWPOS_CENTERED,
		state->config.width,
		state->config.height,
		SDL_WINDOW_SHOWN
	);
	if (window == NULL) {
//		fprintf_s(stderr, "Window could not be created! SDL_Error: %s\n", SDL_GetError());
		return 1;
	}
	state->window = window;
	state->renderer = SDL_CreateRenderer(window, -1, SDL_RENDERER_ACCELERATED);

	return 0;
}

void IZ_VideoTeardown(IZ_VideoState* state) {
	SDL_DestroyWindow(state->window);
}
